package com.yifan.yf_ads.activity;

import android.content.Context;
import android.content.Intent;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowInsets;
import android.widget.ImageView;

import androidx.fragment.app.FragmentActivity;

import com.bumptech.glide.Glide;
import com.yfanads.android.YFAdsManager;
import com.yifan.yf_ads.R;
import com.yifan.yf_ads.admanager.SplashAdManager;
import com.yifan.yf_ads.utils.Constants;
import com.yifan.yf_ads.utils.DeviceUtils;
import com.yifan.yf_ads.utils.FLog;

import java.util.Arrays;
import java.util.List;

/**
 * 一句话描述该类功能.
 *
 * @author JamesQian
 * @version 1.0
 * @copyright 亿帆
 * @date 2024/1/24 14:19
 **/
public class YFSplashActivity extends FragmentActivity implements SplashCallMgr.SplashAdCall {
    private static final String TAG = "YFSplashActivity";
    private static final long TIMEOUT = 5000;// 单位：毫秒 建议设置为3000至5000
    private static final int SKIP_SECOND = 5;// 单位：秒

    private static final boolean IS_SHOW_STATUS_BAR = true; //是否显示状态栏，用来计算Activity高度

    private int optimalHeightWithPX;
    private ViewGroup container; //开屏广告容器

    private ImageView defaultBackgroundImage;
    private String destActivityFullPath;

    private Handler mHandler;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        SplashCallMgr.setSplashAdCall(this);
        SplashAdManager.isLeftActivity = false;
        setContentView(R.layout.yf_flutter_activity_ad_splash);
        setFullScreen();
        // 注意，container的状态必须是VISIBLE的，否则无法加载广告
        // 并且container的高度必须大于等于屏幕高度的75%，否则广点通广告无效展示
        container = findViewById(R.id.id_splash_container);
        defaultBackgroundImage = findViewById(R.id.default_background);
        findDefaultBackgroundImage();
        optimalHeightWithPX = getActivityHeight();
        destActivityFullPath = getIntent().getStringExtra(Constants.KEY_DEST_ACTIVITY_FULL_PATH);
        mHandler = new Handler();
        fetchAd();
    }

    private void findDefaultBackgroundImage() {
        try {
            AssetManager am = getAssets();
            List<String> mapList = Arrays.asList(am.list(""));
            if (mapList.size() == 0) {
                FLog.i(TAG, "mapList == null");
                return;
            }
            if (!mapList.contains("default_splash_back.png")) {
                FLog.i(TAG, "mapList.contains false");
                return;
            }
            defaultBackgroundImage.setVisibility(View.VISIBLE);
            Glide.with(this)
                    .load(Uri.parse("file:///android_asset/default_splash_back.png"))
                    .into(defaultBackgroundImage);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        FLog.i(TAG, "onPause.");
        SplashAdManager.isLeftActivity = true;
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (SplashAdManager.isClicked || SplashAdManager.isAdDismiss) {
            FLog.i(TAG, "onResume");
            gotoMain();
        }
        SplashAdManager.isLeftActivity = false;
    }

    void fetchAd() {
        if (!YFAdsManager.getInstance().isInitSuc()) {
            FLog.e(TAG, "YFAdSDK is not init");
            gotoMain();
        }
        defaultBackgroundImage.setVisibility(View.GONE);

        sendSendMsg();
        autoCancelMsg();
    }

    private void autoCancelMsg() {
        if (mHandler != null) {
            mHandler.postDelayed(() -> {
                if (SplashCallMgr.getAdManagerIF() != null
                        && SplashCallMgr.getAdManagerIF().isLoadSuccess()) {
                    gotoMain();
                }
            }, 3000);
        }
    }

    private void sendSendMsg() {
        if (mHandler != null) {
            mHandler.postDelayed(() -> {
                if (SplashCallMgr.getAdManagerIF() != null) {
                    SplashCallMgr.getAdManagerIF().showAds(YFSplashActivity.this, container);
                }
            }, 100);
        }
    }

    public void removeHandler() {
        if (mHandler != null) {
            mHandler.removeCallbacksAndMessages(null);
            mHandler = null;
        }
    }

    @Override
    public void gotoMain() {
        if (!TextUtils.isEmpty(destActivityFullPath)) {
            try {
                Class<?> destActivityClass = Class.forName(destActivityFullPath);
                Intent intent = new Intent(this, destActivityClass);
                this.startActivity(intent);

            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
        }
        finish();
        FLog.i("gotoMain finish");
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (SplashCallMgr.getAdManagerIF() != null) {
            SplashCallMgr.getAdManagerIF().releaseAd();
        }
    }

    /**
     * 开屏页一定要禁止用户对返回按钮的控制，否则将可能导致用户手动退出了App而广告无法正常曝光和计费
     */
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK || keyCode == KeyEvent.KEYCODE_HOME) {
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    private int getActivityHeight() {
        int activityHeight = 0;
        try {
            int statusBarHeight = getStatusBarHeight(this);
            activityHeight = DeviceUtils.getScreenHeight(this);
            if (IS_SHOW_STATUS_BAR) {
                //如果显示状态栏，Activity的高度就是屏幕高度-状态栏高度
                activityHeight = activityHeight - statusBarHeight;
            }
            FLog.i(TAG, "resetLayoutSize statusBarHeight:" + statusBarHeight + ", activityHeight:" + activityHeight);
        } catch (Exception exception) {
            exception.printStackTrace();
        }
        return activityHeight;
    }

    /**
     * 获取状态栏高度
     *
     * @param context
     * @return
     */
    private static int getStatusBarHeight(Context context) {
        Resources resources = context.getResources();
        int resourceId = resources.getIdentifier("status_bar_height", "dimen", "android");
        int height = resources.getDimensionPixelSize(resourceId);
        return height;
    }

    private void setFullScreen() {
        if (Build.VERSION.SDK_INT >= 30) {
            //Android11 适配
            /**
             * 如果编译的sdk是30以上，直接使用如下代码。
             */
            getWindow().getDecorView().getWindowInsetsController().hide(
                    WindowInsets.Type.statusBars() | WindowInsets.Type.navigationBars());

            /**
             * 如果编译的sdk小于30，使用如下反射的方式。
             *
             * 由于Android版本限制，可能会遇到错误(建议还是使用大于30的sdk编译)：
             * Error: Reflective access to getWindowInsetsController is forbidden when targeting API 29 and above [BlockedPrivateApi]
             *  Explanation for issues of type "BlockedPrivateApi":
             *    Usage of restricted non-SDK interface is forbidden for this targetSDK.
             *    Accessing non-SDK methods or fields through reflection has a high
             *    likelihood to break your app between versions, and is being restricted to
             *    facilitate future app compatibility.
             */
//            try {
//                View decorView = getWindow().getDecorView();
//                String controll = "get" + "Window" + "Insets" + "Controller";
//                // Reflective access to getWindowInsetsController is forbidden when targeting API 29 and above [BlockedPrivateApi]
//                Method getController = View.class.getDeclaredMethod(controll);
//                getController.setAccessible(true);
//                Object insetsController = getController.invoke(decorView);
//                Class<?> WindowInsetsController = Class.forName("android.view.WindowInsetsController");
//                Method hide = WindowInsetsController.getMethod("hide", int.class);
//                hide.setAccessible(true);
//                hide.invoke(insetsController, 3);
//            } catch (Throwable e) {
//                e.printStackTrace();
//            }
        } else {
            // Note that some of these constants are new as of API 16 (Jelly Bean)
            // and API 19 (KitKat). It is safe to use them, as they are inlined
            // at compile-time and do nothing on earlier devices.
            getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LOW_PROFILE
                    | View.SYSTEM_UI_FLAG_FULLSCREEN
                    | View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                    | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION);
        }
    }
}
