import 'dart:core';
import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:yf_ads/callbacks/yf_ads_callback.dart';
import 'package:yf_ads/sizes/yf_ads_banner_android_size.dart';
import 'package:yf_ads/sizes/yf_ads_banner_ios_size.dart';
import 'package:yf_ads/sizes/yf_ads_banner_size.dart';
import 'package:yf_ads/yf_ads.dart';
import 'package:yf_ads/ads/yf_ads_banner_view.dart';
import 'package:yf_ads/bean/yf_ads_banner_bean.dart';
import 'package:yf_ads/bean/yf_ads_banner_bean.dart';
import 'package:yf_ads/managers/yf_banner.dart';
import  'dart:ui';
import 'dart:convert';

class YFAdsBannerViewDemo extends StatefulWidget {


  @override
  State<StatefulWidget> createState() {
    return YFAdsBannerViewDemoState();
  }
}

class YFAdsBannerViewDemoState extends State<YFAdsBannerViewDemo> {
  bool showBanner = false;
  var index = "banner";

  double? width = 300;
  double? height =  600.0;

  final viewType = "com.yf/banner_platform_view";


  void bannerViewCallBack(YFAdsCallBack args) {
    print("bannerViewCallBack is ${args.method}");


    if (args.method == "onDislike" || args.method == "yfBannerAdClosed") {
      setState(() {
        showBanner = false;
      });
    }
  }

  void showBannerAd(){
    width = MediaQuery.of(context).size.width;
    setState(() {
      showBanner = true;
    });
  }

  void reloadBannerAd() {
    print("reloadBannerAd");
    YFAds.instance.clearBannerPageCache();
    width = MediaQuery.of(context).size.width;

    // 以下数据和view是一致的
    int adSizeIndex = 0;
    BannerAdSize bannerAdSizeImpl;
    Size calSize;

    var positionId;
    var adSizeType;
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      positionId = "2eb399cceb2b4003ad9fe5942e12b30d";
      // bannerAdSizeImpl = IOSBannerAdSize();
      // adSizeType = IOSBannerAdType.AD_SIZE_NEW_H800;
      // adSizeIndex =
      //     IOSBannerAdType.values.indexOf(adSizeType! as IOSBannerAdType);

      calSize = Size(this.width!,this.height!);
    } else {
      positionId = "b70687e8-1eb8-4615-88df-65858fe70842";
      adSizeType = AndroidBannerAdType.AD_SIZE_W600xH400;
      bannerAdSizeImpl = AndroidBannerAdSize();
      adSizeIndex = AndroidBannerAdType.values
          .indexOf(adSizeType! as AndroidBannerAdType);
      calSize = _getSize(context, adSizeIndex, false, bannerAdSizeImpl);

    }
    print(
        "requestBannerAd. width: ${calSize.width} and height: ${calSize.height} and adSizeIndex: $adSizeIndex");
    var reaperBannerBean = YFAdsBannerBean(
        uniqueKey: index,
        //广告宽
        width: calSize.width.toInt(),
        //广告高
        height: calSize.height.toInt(),
        //是否展示关闭按钮
        showDislike: true,
        //banner尺寸
        adSizeIndex: adSizeIndex,
        //广告背景圆角
        cornerSize: 30,
        //当前广告位是否是爱奇艺广告
        loadAndShow: false,
        //广告位id
        positionId: positionId,
        callBack: bannerViewCallBack);

    var createParams = jsonEncode(reaperBannerBean);
    print("createParams is $createParams");
    YFBannerManager.loadBannerAd(reaperBannerBean);

  }

  Size _getSize(BuildContext context, int adSizeIndex, bool expressAd,
      BannerAdSize bannerAdSizeImpl) {
    double screenWidth = MediaQuery.of(context).size.width;
    if (!expressAd) {
      double width =
      bannerAdSizeImpl.getWidth(adSizeIndex, screenWidth, 12);
      double height =
      bannerAdSizeImpl.getHeight(adSizeIndex, width, 0, false);
      return Size(width, height);
    }
    return bannerAdSizeImpl.getExpressBannerAdWidth(
        adSizeIndex, screenWidth, 12, false);
  }

  @override
  Widget build(BuildContext context) {
    print("banner view demo build");


    var positionId;
    var androidOS = false;
    if (defaultTargetPlatform == TargetPlatform.android) {
      positionId = "b70687e8-1eb8-4615-88df-65858fe70842";
      // jd
      // positionId = "a6235f84-25b5-44ee-bf39-395ad7fd1850";
      androidOS = true;
    } else if (defaultTargetPlatform == TargetPlatform.iOS) {
      positionId = "2eb399cceb2b4003ad9fe5942e12b30d";
      androidOS = false;
    }

    var reloadBannerView = Align(
        alignment: Alignment.center,
        child:Column(
          children: [
            ElevatedButton(
              onPressed: reloadBannerAd,
              child: const Text("加载banner广告"),
            ),

            ElevatedButton(
              onPressed: showBannerAd,
              child: const Text("展示 banner广告"),
            )

          ],
        )
      );

    var bannerAdView = showBanner
        ? YFAdsBannerView(
            // key: Key("$bannerCornerSize"),
            uniqueId: index,
            width: this.width,
            height: this.height,
            onAdCallBack: bannerViewCallBack,
            topMarginSize: 36,
            positionId: positionId,
            cornerSize: 30,
            adSizeType: androidOS
                ? AndroidBannerAdType.AD_SIZE_W600xH400
                : IOSBannerAdType.AD_SIZE_NEW_H800,
          )
        : reloadBannerView;
    return Scaffold(
      appBar: AppBar(
        title: const Text('Banner广告'),
      ),
      body: bannerAdView,
    );
  }

  @override
  void dispose() {
    super.dispose();
    print("YFAdsBannerViewDemo dispose");
    YFAds.instance.clearBannerPageCache();
  }
}
