
#import <Foundation/Foundation.h>
#import "YFAdsBannerViewFactory.h"
#import "YFAdsUtils.h"
#import "YFAdsBannerView.h"
#import "YFAdsConstant.h"
#import "YFAdsSDKManager.h"


@interface YFAdsBannerViewFactory ()
@property (nonatomic, strong) FlutterEventSink flutterEventSink;
@property (nonatomic, strong) UIViewController *viewController;
@property (nonatomic, strong) NSMutableDictionary *bannerViewCachedDic;
@property (nonatomic, strong) NSMutableDictionary *parameters;

@end

@implementation YFAdsBannerViewFactory

- (instancetype)initViewFactory {
    self = [super init];
    if (self) {
        _bannerViewCachedDic = [NSMutableDictionary new];
    }
    return self;
}

- (void)setFlutterEventSink:(FlutterEventSink)flutterEventSink{
    _flutterEventSink = flutterEventSink;
}

- (void)setFlutterViewController:(UIViewController *)viewController{
    _viewController = viewController;
}

- (NSObject<FlutterPlatformView>*)createWithFrame:(CGRect)frame
                                   viewIdentifier:(int64_t)viewId
                                        arguments:(id _Nullable)args {
    NSDictionary *parameters = [YFAdsUtils jsonString2Dictionary:args];
    self.parameters = parameters;
    YFAdsBannerView *bannerView = nil;
    NSString *uniqueId = parameters[KEY_UNIQUE_KEY];
    bannerView = [[YFAdsSDKManager shareInstance].bannerViews objectForKey:uniqueId];
    if(bannerView){
        NSLog(@"bannerViewFactory createWithFrame hit the cache view and uniqueId %@",uniqueId);
        return bannerView;
    }else {
        NSLog(@"bannerViewFactory createWithFrame uniqueId %@",uniqueId);
        
        if(parameters[@"loadAndShow"] && ![parameters[@"loadAndShow"] isKindOfClass:NSNull.class]) {
            NSInteger isLoadAndShow = [parameters[@"loadAndShow"] integerValue];
            if (isLoadAndShow == 1) {
                if(parameters){
                    bannerView = [[YFAdsBannerView alloc] initWithFrame:frame arguments:parameters eventSink:_flutterEventSink viewController:_viewController];
                    return bannerView;
                }
            }
        }
    }

    return bannerView;
}


- (NSObject<FlutterMessageCodec>*)createArgsCodec{
    return [FlutterStandardMessageCodec sharedInstance];
}

- (void)bannerViewPageFinished{
    
    if(_bannerViewCachedDic){
        NSLog(@"BannerViewFactory bannerViewPageFinished");

        if(_parameters[KEY_UNIQUE_KEY] && ![_parameters[@"KEY_UNIQUE_KEY"] isKindOfClass:NSNull.class]){
            [[YFAdsSDKManager shareInstance].bannerViews removeObjectForKey:_parameters[KEY_UNIQUE_KEY]];
        }
        [_bannerViewCachedDic removeAllObjects];
    }
}

@end
