
import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/services.dart';
import 'package:yf_ads/bean/yf_ads_banner_bean.dart';
import 'package:yf_ads/bean/yf_ads_init_sdk_base_bean.dart';
import 'package:yf_ads/bean/yf_ads_interaction_bean.dart';
import 'package:yf_ads/bean/yf_ads_reward_video_bean.dart';
import 'package:yf_ads/bean/yf_ads_splash_bean.dart';
import 'package:yf_ads/callbacks/yf_ads_callback.dart';
import 'package:yf_ads/callbacks/yf_ads_draw_native_express_video_callback.dart';
import 'package:yf_ads/callbacks/yf_ads_express_feed_callback.dart';
import 'package:yf_ads/callbacks/yf_ads_native_draw_feed_video_callback.dart';
import 'package:yf_ads/callbacks/yf_ads_reward_video_callback.dart';

import 'bean/yf_ads_express_feed_bean.dart';

class YFAds {

  static const MethodChannel channel = MethodChannel('yf_ads_plugin');
  static const EventChannel eventChannel =  EventChannel('yf_ads_plugin_event');

  YFAds._privateConstructor() {
    eventChannel.receiveBroadcastStream().listen(_onEvent, onError: _onError);
  }

  static YFAds get instance => _instance;
  static final YFAds _instance = YFAds._privateConstructor();
  /// for platform call back
  static Function? callBack;
  void _onEvent(Object? event) {
    print("_onEvent is invoke $event callBack $callBack");
    if (callBack != null) {
      String reaperAdCallBackStr = event as String;
      try {
        var callBackResultMap = jsonDecode(reaperAdCallBackStr);
        YFAdsCallBack reaperAdCallBack;
        if (reaperAdCallBackStr.contains(YFAdsCallBack.YF_ADS_TYPE_REWARD_VIDEO)) {
          reaperAdCallBack =
              YFAdsRewardVideoCallBack.fromJson(callBackResultMap);
        } else if (reaperAdCallBackStr
            .contains(YFAdsCallBack.YF_ADS_TYPE_EXPRESS_FEED)) {
          reaperAdCallBack =
              YFAdsExpressFeedCallBack.fromJson(callBackResultMap);
        } else if (reaperAdCallBackStr
            .contains(YFAdsCallBack.YF_ADS_TYPE_DRAW_NATIVE_EXPRESS_VIDEO)) {
          reaperAdCallBack =
              YFAdsDrawNativeExpressVideoCallBack.fromJson(callBackResultMap);
        } else if (reaperAdCallBackStr
            .contains(YFAdsCallBack.YF_ADS_TYPE_NATIVE_DRAW_FEED_VIDEO)) {
          reaperAdCallBack =
              YFAdsNativeDrawFeedVideoCallBack.fromJson(callBackResultMap);
        } else {
          reaperAdCallBack = YFAdsCallBack.fromJson(callBackResultMap);
        }
        print("callBackResult type is ${reaperAdCallBack.adType}");
        callBack!(reaperAdCallBack);
      } on FormatException {
        print("_onEvent call data is not json");
      }
    }
  }

  void _onError(Object? error) {
    print("_onError is invoke $error");
    if (callBack != null) {
      if (Platform.isIOS) {
        callBack!("$error");
      } else {
        callBack!(error);
      }
    }
  }

  static const KEY_METHOD_INIT_SDK = "initAdsSDK";
  static const KEY_METHOD_REQUEST_REWARD_VIDEO_AD = "requestRewardVideoAd";
  static const KEY_METHOD_REQUEST_INTERACTION_AD = "requestInteractionAd";
  static const KEY_METHOD_SPLASH_AD = "requestSplashAd";
  static const KEY_METHOD_EXPRESS_AD = "requestExpressAd";

  static const KEY_METHOD_BANNER_AD = "requestBannerAd";
  static const KEY_METHOD_BANNER_PAGE_CLOSED = "notifyBannerPageClosed";
  static const KEY_METHOD_NATIVE_PAGE_CLOSED = "notifyNativeAdPageClosed";
  static const KEY_METHOD_EXPRESSFEED_PAGE_CLOSED =
      "notifyExpressFeedAdPageClosed";
  static const KEY_METHOD_DRAW_NATIVE_EXPRESS_VIDEO_PAGE_CLOSED =
      "notifyDrawNativeExpressVideoAdPageClosed";
  static const KEY_METHOD_NATIVE_DRAW_FEED_VIDEO_AD_PAGE_CLOSED =
      "notifyNativeDrawFeedVideoAdPageClosed";
  static const KEY_METHOD_STREAM_PAGE_CLOSED = "notifyStreamAdPageClosed";

  Future<void> initAdsSDK(
      YFAdsInitSDKBaseBean reaperInitSdkBaseBean) async {
    final String initParameters = jsonEncode(reaperInitSdkBaseBean);
    await channel.invokeMethod(KEY_METHOD_INIT_SDK, initParameters);
  }

  Future<void> loadRewardVideoAd(
      YFAdsRewardVideoBean reaperRewardVideoBean) async {
    final String rewardVideoParameters = jsonEncode(reaperRewardVideoBean);
    callBack = reaperRewardVideoBean.callBack;
    await channel.invokeMethod(
        KEY_METHOD_REQUEST_REWARD_VIDEO_AD, rewardVideoParameters);
  }

  Future<void> loadInteractionAd(
      YFAdsInteractionBean reaperInteractionBean) async {
    final String interactionAdParameters = jsonEncode(reaperInteractionBean);
    callBack = reaperInteractionBean.callBack;
    await channel.invokeMethod(
        KEY_METHOD_REQUEST_INTERACTION_AD, interactionAdParameters);
  }

  Future<void> loadSplashAd(YFAdsSplashBean reaperSplashBean) async {
    callBack = reaperSplashBean.callBack;
    final String splashParameters = jsonEncode(reaperSplashBean);
    await channel.invokeMethod(KEY_METHOD_SPLASH_AD, splashParameters);
  }

  Future<void> loadBannerAd(YFAdsBannerBean banner) async {
    callBack = banner.callBack;
    final String splashParameters = jsonEncode(banner);
    await channel.invokeMethod(KEY_METHOD_BANNER_AD, splashParameters);
  }

  Future<void> loadExpress(YFAdsExpressFeedBean expressFeedBean) async {
    callBack = expressFeedBean.callBack;
    final String splashParameters = jsonEncode(expressFeedBean);
    await channel.invokeMethod(KEY_METHOD_EXPRESS_AD, splashParameters);
  }


  void setBannerAdCallBackFunction(Function(YFAdsCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  void setNativeAdCallBackFunction(Function(YFAdsCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  void setExpressFeedAdCallBackFunction(
      Function(YFAdsExpressFeedCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  void setDrawNativeExpressVideoAdCallBackFunction(
      Function(YFAdsDrawNativeExpressVideoCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  void setNativeDrawFeedVideoAdCallBackFunction(
      Function(YFAdsNativeDrawFeedVideoCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  Future<void> clearBannerPageCache() async {
    await channel.invokeMethod(KEY_METHOD_BANNER_PAGE_CLOSED);
  }

  Future<void> clearNativePageCache() async {
    await channel.invokeMethod(KEY_METHOD_NATIVE_PAGE_CLOSED);
  }

  Future<void> clearExpressFeedPageCache() async {
    if (Platform.isIOS) {
      await channel.invokeMethod(KEY_METHOD_BANNER_PAGE_CLOSED);
    } else {
      await channel.invokeMethod(KEY_METHOD_EXPRESSFEED_PAGE_CLOSED);
    }
  }

  Future<void> clearDrawNativeExpressVideoPageCache() async {
    await channel
        .invokeMethod(KEY_METHOD_DRAW_NATIVE_EXPRESS_VIDEO_PAGE_CLOSED);
  }

  Future<void> clearNativeDrawFeedVideoPageCache() async {
    await channel
        .invokeMethod(KEY_METHOD_NATIVE_DRAW_FEED_VIDEO_AD_PAGE_CLOSED);
  }

  Future<void> clearStreamPageCache() async {
    await channel.invokeMethod(KEY_METHOD_STREAM_PAGE_CLOSED);
  }
}
